@@ -820,202 +820,234 @@ int PatBlt_32bpp(HGDI_DC hdc, int nXLeft, int nYLeft, int nWidth, int nHeight, i
 	return 1;
 }
 
-INLINE void SetPixel_BLACK_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_BLACK_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = 0 */
 	*pixel = 0;
 }
 
-INLINE void SetPixel_NOTMERGEPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_NOTMERGEPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = ~(D | P) */
 	*pixel = ~(*pixel | *pen);
 }
 
-INLINE void SetPixel_MASKNOTPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_MASKNOTPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = D & ~P */
 	*pixel &= ~(*pen);
 }
 
-INLINE void SetPixel_NOTCOPYPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_NOTCOPYPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = ~P */
 	*pixel = ~(*pen);
 }
 
-INLINE void SetPixel_MASKPENNOT_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_MASKPENNOT_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = P & ~D */
 	*pixel = *pen & ~*pixel;
 }
 
-INLINE void SetPixel_NOT_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_NOT_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = ~D */
 	*pixel = ~(*pixel);
 }
 
-INLINE void SetPixel_XORPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_XORPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = D ^ P */
 	*pixel = *pixel ^ *pen;
 }
 
-INLINE void SetPixel_NOTMASKPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_NOTMASKPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = ~(D & P) */
 	*pixel = ~(*pixel & *pen);
 }
 
-INLINE void SetPixel_MASKPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_MASKPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = D & P */
 	*pixel &= *pen;
 }
 
-INLINE void SetPixel_NOTXORPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_NOTXORPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = ~(D ^ P) */
 	*pixel = ~(*pixel ^ *pen);
 }
 
-INLINE void SetPixel_NOP_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_NOP_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = D */
 }
 
-INLINE void SetPixel_MERGENOTPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_MERGENOTPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = D | ~P */
 	*pixel |= ~(*pen);
 }
 
-INLINE void SetPixel_COPYPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_COPYPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = P */
 	*pixel = *pen;
 }
 
-INLINE void SetPixel_MERGEPENNOT_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_MERGEPENNOT_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = P | ~D */
 	*pixel = *pen | ~(*pixel);
 }
 
-INLINE void SetPixel_MERGEPEN_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_MERGEPEN_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = P | D */
 	*pixel |= *pen;
 }
 
-INLINE void SetPixel_WHITE_32bpp(uint32 *pixel, uint32 *pen)
+INLINE void SetPixel_WHITE_32bpp(uint32* pixel, uint32* pen)
 {
 	/* D = 1 */
 	*pixel = 0xFFFFFF;
 }
 
-pSetPixel32_ROP2 SetPixel32_ROP2_[32] =
+#define PIXEL_TYPE		uint32
+#define GDI_GET_POINTER		gdi_GetPointer_32bpp
+#define GDI_GET_PEN_COLOR	gdi_GetPenColor_32bpp
+
+#define LINE_TO			LineTo_BLACK_32bpp
+#define SET_PIXEL_ROP2		SetPixel_BLACK_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_NOTMERGEPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_NOTMERGEPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_MASKNOTPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_MASKNOTPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_NOTCOPYPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_NOTCOPYPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_MASKPENNOT_32bpp
+#define SET_PIXEL_ROP2		SetPixel_MASKPENNOT_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_NOT_32bpp
+#define SET_PIXEL_ROP2		SetPixel_NOT_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_XORPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_XORPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_NOTMASKPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_NOTMASKPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_MASKPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_MASKPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_NOTXORPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_NOTXORPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_NOP_32bpp
+#define SET_PIXEL_ROP2		SetPixel_NOP_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_MERGENOTPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_MERGENOTPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_COPYPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_COPYPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_MERGEPENNOT_32bpp
+#define SET_PIXEL_ROP2		SetPixel_MERGEPENNOT_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_MERGEPEN_32bpp
+#define SET_PIXEL_ROP2		SetPixel_MERGEPEN_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#define LINE_TO			LineTo_WHITE_32bpp
+#define SET_PIXEL_ROP2		SetPixel_WHITE_32bpp
+#include "include/line.c"
+#undef LINE_TO
+#undef SET_PIXEL_ROP2
+
+#undef PIXEL_TYPE
+#undef GDI_GET_POINTER
+#undef GDI_GET_PEN_COLOR
+
+pLineTo_32bpp LineTo_ROP2_32bpp[32] =
 {
-	SetPixel_BLACK_32bpp,
-	SetPixel_NOTMERGEPEN_32bpp,
-	SetPixel_MASKNOTPEN_32bpp,
-	SetPixel_NOTCOPYPEN_32bpp,
-	SetPixel_MASKPENNOT_32bpp,
-	SetPixel_NOT_32bpp,
-	SetPixel_XORPEN_32bpp,
-	SetPixel_NOTMASKPEN_32bpp,
-	SetPixel_MASKPEN_32bpp,
-	SetPixel_NOTXORPEN_32bpp,
-	SetPixel_NOP_32bpp,
-	SetPixel_MERGENOTPEN_32bpp,
-	SetPixel_COPYPEN_32bpp,
-	SetPixel_MERGEPENNOT_32bpp,
-	SetPixel_MERGEPEN_32bpp,
-	SetPixel_WHITE_32bpp
+	LineTo_BLACK_32bpp,
+	LineTo_NOTMERGEPEN_32bpp,
+	LineTo_MASKNOTPEN_32bpp,
+	LineTo_NOTCOPYPEN_32bpp,
+	LineTo_MASKPENNOT_32bpp,
+	LineTo_NOT_32bpp,
+	LineTo_XORPEN_32bpp,
+	LineTo_NOTMASKPEN_32bpp,
+	LineTo_MASKPEN_32bpp,
+	LineTo_NOTXORPEN_32bpp,
+	LineTo_NOP_32bpp,
+	LineTo_MERGENOTPEN_32bpp,
+	LineTo_COPYPEN_32bpp,
+	LineTo_MERGEPENNOT_32bpp,
+	LineTo_MERGEPEN_32bpp,
+	LineTo_WHITE_32bpp
 };
 
 int LineTo_32bpp(HGDI_DC hdc, int nXEnd, int nYEnd)
 {
-	int x, y;
-	int x1, y1;
-	int x2, y2;
-	int e, e2;
-	int dx, dy;
-	int sx, sy;
-	HGDI_BITMAP bmp;
-	int bx1, by1;
-	int bx2, by2;
+	pLineTo_32bpp _LineTo;
+	int rop2 = gdi_GetROP2(hdc) - 1;
 
-	int irop2;
-	uint32 pen;
-	uint32 *pixel;
+	_LineTo = LineTo_ROP2_32bpp[rop2];
 
-	x1 = hdc->pen->posX;
-	y1 = hdc->pen->posY;
-	x2 = nXEnd;
-	y2 = nYEnd;
-
-	dx = (x1 > x2) ? x1 - x2 : x2 - x1;
-	dy = (y1 > y2) ? y1 - y2 : y2 - y1;
-
-	sx = (x1 < x2) ? 1 : -1;
-	sy = (y1 < y2) ? 1 : -1;
-
-	e = dx - dy;
-
-	x = x1;
-	y = y1;
-
-	irop2 = gdi_GetROP2(hdc) - 1;
-	bmp = (HGDI_BITMAP) hdc->selectedObject;
-
-	if (hdc->clip->null)
-	{
-		bx1 = (x1 < x2) ? x1 : x2;
-		by1 = (y1 < y2) ? y1 : y2;
-		bx2 = (x1 > x2) ? x1 : x2;
-		by2 = (y1 > y2) ? y1 : y2;
-	}
+	if (_LineTo != NULL)
+		return _LineTo(hdc, nXEnd, nYEnd);
 	else
-	{
-		bx1 = hdc->clip->x;
-		by1 = hdc->clip->y;
-		bx2 = bx1 + hdc->clip->w - 1;
-		by2 = by1 + hdc->clip->h - 1;
-	}
-
-	pen = gdi_GetPenColor_32bpp(hdc->pen);
-
-	while (1)
-	{
-		if (!(x == x2 && y == y2))
-		{
-			if ((x >= bx1 && x <= bx2) && (y >= by1 && y <= by2))
-			{
-				pixel = gdi_GetPointer_32bpp(bmp, x, y);
-				SetPixel32_ROP2_[irop2](pixel, &pen);
-			}
-		}
-		else
-		{
-			break;
-		}
-
-		e2 = 2 * e;
-
-		if (e2 > -dy)
-		{
-			e -= dy;
-			x += sx;
-		}
-
-		if (e2 < dx)
-		{
-			e += dx;
-			y += sy;
-		}
-	}
-
-	return 1;
+		return 0;
 }
